@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 from collections import defaultdict
 from dataclasses import dataclass, fields
 from itertools import chain, filterfalse
@@ -8,10 +6,13 @@ from typing import (
     Callable,
     Collection,
     Iterable,
+    Iterator,
     Mapping,
+    Optional,
     Set,
     Tuple,
     TypeVar,
+    overload,
 )
 
 flatten = chain.from_iterable
@@ -58,8 +59,45 @@ class compose:
         return value
 
 
-# Adapted from dataclasses.dataclass_tools
-# Under Apache license 2.0
+# I'd like for this to be a proper Protocol,
+# but mypy won't allow `object` to be recognized as such.
+# So let's just go with object.
+SupportsBool = object
+Predicate = Callable[[_T1], SupportsBool]
+
+
+def both(__a: Predicate[_T1], __b: Predicate[_T1]) -> Predicate[_T1]:
+    return lambda x: __a(x) and __b(x)
+
+
+@overload
+def either(
+    __a: Callable[[_T1], bool], __b: Callable[[_T1], bool]
+) -> Callable[[_T1], bool]:
+    ...
+
+
+@overload
+def either(__a: Predicate[_T1], __b: Predicate[_T1]) -> Predicate[_T1]:
+    ...
+
+
+def either(__a: Any, __b: Any) -> Any:
+    return lambda x: __a(x) or __b(x)
+
+
+def map_optional(
+    f: Callable[[_T1], Optional[_T2]], it: Iterable[_T1]
+) -> Iterator[_T2]:
+    return filterfalse(_is_none, map(f, it))  # type: ignore
+
+
+def _is_none(x: object) -> bool:
+    return x is None
+
+
+# From https://github.com/ericvsmith/dataclasses/blob/master/dataclass_tools.py
+# License: https://github.com/ericvsmith/dataclasses/blob/master/LICENSE.txt
 # Changed only `dataclass.fields` naming
 def add_slots(cls: _Ttype) -> _Ttype:  # pragma: no cover
     # Need to create a new class, since we can't set __slots__
